% TD_APROX
%
%   Function TD_APROX approximates transport delay.
%
%   [num_ap,den_ap] = td_aprox(num,den,td)
%
%   or
%
%   [num_ap,den_ap] = td_aprox(num,den,td,order,method)
%
%   where
%
%   num_ap  - is output vector of approximated numerator of system
%   den_ap  - is output vector of approximated denominator of system
%   num     - is input vector of numerator of system
%   den     - is input vector of denominator of system
%   td      - is input value of transport delay
%   order   - is input value of order of approximation formula - default 1
%   method  - is input string of approximation method
%           METHOD = 'macl' - for Maclauri approximation - default
%           METHOD = 'tayn' - for Taylor expansion of numerator
%           METHOD = 'tayd' - for Taylor expansion of denominator
%           METHOD = 'pade' - for Pade approximation
%           METHOD = 'limi' - for limit approximation of transport delay
%
%   juraj.oravec@stuba.sk
%
%   2011.06.01.
%
function [num_ap,den_ap] = td_aprox(num,den,td,order,method)

% Initial value of outputs
%
num_td = [1];
den_td = [1];

% Set default method
%
if (nargin < 5)
    
    method = 'macl'; % Default method - Maclauri
    
else
    
    % Get first 4 characters of method name
    %
    method = method(1:4);
    
end

% Set default order
%
if (nargin < 4)
    order = 1; % Default method - 1st
end


% Maclauri
%
if method == 'macl'

    num_td = [1];
    den_td = [1];
    
    for k = 1 : order
    
        num_part = (-1)^k*((td/2)^(k))/factorial(k);
        den_part = ((td/2)^(k))/factorial(k);
        num_td = [num_part,num_td];
        den_td = [den_part,den_td];
    
    end
    
% Taylor - numerator
%
elseif method == 'tayn'
    
    num_td = [1];
    den_td = [1];
    
    for k = 1 : order
    
        num_part = (-1)^k*(td^(k))/factorial(k);
        num_td = [num_part,num_td];
    
    end

% Taylor - denominator
%
elseif method == 'tayd'
    
    num_td = [1];
    den_td = [1];
    
    for k = 1 : order
    
        den_part = (td^(k))/factorial(k);
        den_td = [den_part,den_td];
    
    end

% Pade
%
elseif method == 'pade'
    
    num_td = [];
    den_td = [];
    
    for k = 0 : order
    
        num_part = factorial(2*order - k)/(factorial(k)*factorial(order-k))*(-td)^(k);
        num_td = [num_part,num_td];
        den_part = factorial(2*order - k)/(factorial(k)*factorial(order-k))*td^(k);
        den_td = [den_part,den_td];
    
    end

% Limit aproximation
%
elseif method == 'limi'
    
    num_td = [1];
    den_td = [td/order 1];
    
    for k = 1 : order-1
    
        den_td = conv(den_td,[td/order 1]);
    
    end

% --- %

end     % if

num_ap = conv(num,num_td);
den_ap = conv(den,den_td);
